CharWindow {
	var
		index,
		lines,
		window,
		rows,
		cols,

		oscResps;
		
	*new {
		arg
			numPlayers,
			projection = false,
			thisScreenWidth = 1280,
			projectionScreenWidth = 1280,
			displayHeight = 720;
			
		^super.new.init(
			numPlayers,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight
		);
	}
	
	init {
		arg
			numPlayers,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight;
		var
			bounds;
		
		
		rows = floor(displayHeight / 60);           // 12 rows of chars for 720 pixels high
		cols = floor(thisScreenWidth * 182 / 1280); // 182 columns of chars for 1280 pixels wide

		index = [0, 0];
		lines = all {: "", i <- (1..rows) };			
		
		this.clear_lines;
		// breaking the text into lines prevents ugly overflows into the next line
		if(
			projection,
			{
				bounds = Rect(
					thisScreenWidth,
					displayHeight * 2 / 3,
					projectionScreenWidth,
					displayHeight / 3
				);
			},
			{
				bounds = Rect(
					0,
					displayHeight * 2 / 3,
					thisScreenWidth,
					displayHeight / 3
				)
			}
		);
		
		window = Window(
			"CharWindow",
			bounds,
			resizable: false,
			border: false;
		);

		window.view.background = Color.black;
		window.front;
		CmdPeriod	.add({window.close});
		
		window.drawHook = {
			Pen.font = Font("Monaco", 12);
			Pen.color = Color.green;

			lines.do {
				arg i, index;
				var rowHeight = 15;

				Pen.stringInRect(
					i,
					Rect(0, index * rowHeight, bounds.width, rowHeight)
				);
			};
		};
		

		/* OSC */
		try {
			this.init_osc_responders(numPlayers);
		}
		{
			"WARNING: KeyboardWindow couldn't find all the players in the network.".postln;
		};
		
	}
	
	clear_lines {
	
		rows.do {
			arg i;
			cols.do {
				lines[i] = lines[i] ++ " ";
			}
		};
		
	}
	
	init_osc_responders {
		arg numPlayers;

		// an OSCresponderNode for each player.
		// Players are "0.local", "1.local", "2.local", etc.
		oscResps = Array.newClear(numPlayers);

		numPlayers.do {
			arg i;
			oscResps[i] = OSCresponderNode(
				NetAddr(
					("%.local").format(i.asString),
					57120
				),
				'/charWindow',
				{
					arg
						time,
						resp,
						msg;
					var
						playerNum,
						char;
										
					char = msg[1].asString[0]; // expect a 1-character Symbol,
					                           // convert to a String (Array of Chars),
					                           // read the first Char					
					this.add_char(char);
				}
			).add;
			
		// Remove the OSC responder when done.
		CmdPeriod	.add({oscResps[i].remove});
			
		}
	}
	
	add_char {
		arg char;
		
		lines[index[0]][index[1]] = char;

		if(
			index[1] == (cols - 1),
			{
				index[0] = index[0] + 1 % rows;
				index[1] = 0;
			},
			{
				index[1] = index[1] + 1;
			}
		);


		{window.refresh}.defer;
	}
	
	add_str {
		arg str;
		str.do {
			arg char;
			this.add_char(char)
		}
	}

}