FreqTablesWindow {

	var
		freqTables,
		oscResp,
		window;
		
	*new {
		arg
			numPlayers,
			projection = false,
			thisScreenWidth = 1280,
			projectionScreenWidth = 1280,
			displayHeight = 720;
			
			
		^super.new.init(
			numPlayers,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight
		);
	}
	
	init {
		arg
			numPlayers,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight;
		var
			bounds;

		this.init_osc_responder;

		if(
			numPlayers < 2,
			{
				(
					"WARNING: for FreqTablesWindow to display properly, " ++
					"the `numPlayers' argument must be at least 2.  " ++
					"I'm changing `numPlayers' to 2."
				).postln;
				numPlayers = 2;
			}
		);
		
		/* Initialize the freqTables. */
		freqTables = FreqTables(
			freqs: 4410 ! numPlayers,
			amps: 0 ! numPlayers
		);

		/* Set up the window. */
		if(
			projection,
			{
				bounds = Rect(
					thisScreenWidth,
					displayHeight / 3,
					projectionScreenWidth,
					displayHeight / 3
				)
			},
			// Single-monitor display.
			// Load this window before the others so that it doesn't cover them up.
			{
				bounds = Rect(
					0,
					displayHeight / 3,
					thisScreenWidth,
					displayHeight / 3
				)
			}
		);

		window = Window.new(
			"FreqTablesWindow",
			bounds,
			resizable: false,
			border: false
		);
		
		window.view.background = Color.black;
		window.front;
		CmdPeriod	.add({window.close});		

		
		window.drawHook = {
			var
				h = window.bounds.height,
				w = window.bounds.width,
				tableLabel1,
				tableLabel2;
			
			this.draw_table(
				freqTables.freqTable,
				freqTables.ampTable,
				Rect(2, 0, 0.5 * w, h - 2)
			);

			this.draw_table(
				freqTables.glitchTable,
				freqTables.ampTable,
				Rect(0.5 * w + 2, 0, 0.5 * w, h - 2)
			);		
		}

	
	}

	init_osc_responder {

		oscResp = OSCresponderNode(
			NetAddr("0.local", 57120), // address of conductor
			'/freqTables',
			{
				arg
					time,
					resp,
					msg;
				var
					player,
					parameter,
					value;

				player = msg[1];
				parameter = msg[2];
				value = msg[3];
				
				parameter.switch(
					\amp, {this.set_amp(player, value)},
					\impulseFreq, {this.set_freq(player, value)},
					{
						"Warning: SynthWindow doesn't understand this parameter: " ++
						parameter.asString
					}
				);
			}
		).add;
		
		// Remove the OSC responder when done.
		CmdPeriod	.add({oscResp.remove});
				
	}

	set_pen_color {
		arg i;

		if(
			i == 0,		
			{
				Pen.color = Color.black;
			},
			{
				Pen.color = Color.green;
			}
		)
	}	

	draw_table {
		arg
			freqTable,  // an Array2D of freqs
			ampTable,   // an Array2D of amps
			bounds,     // a Rect within which to draw the table
			label = ""; // a table label

		var
			n = freqTable.cols,
			w = bounds.width,
			h = bounds.height,
			str;
		
		
		Pen.use {
			Pen.translate(bounds.left, bounds.top);
			Pen.font = Font("Monaco", 12);
			Pen.color = Color.green;

			/* Draw a border around the differences. */
			Pen.addRect(
				Rect(
					0,
					h / n,
					(n - 1) * w / n,
					(n - 1) * h / n
				)
			);
			Pen.stroke;
	
			n.do{
				arg i;
				n.do{
					arg j;
					if(
						freqTable[i, j] != nil,
						{
							str = freqTable[i, j].round(0.01).asString;
							this.set_pen_color(ampTable[i, j]);
							Pen.stringAtPoint(
								str.pad_floatString,								Point(
									j * w / n,
									i * h / n
								)
							)
						}
					)
				}
			};
			
			Pen.color = Color.green;
			Pen.stringAtPoint(
				label,
				Point(
					0,
					h
				)
			)
		};

	}
	
	set_freq {
		arg i, val;
		freqTables.set_freq(i, val);
		{window.refresh}.defer;
	}
	
	set_amp {
		arg i, val;
		freqTables.set_amp(i, val);
		{window.refresh}.defer;
	}

}
