KeyboardWindow {
	var
		<>window,
		<>oscResps,
		<>keyboardViews;
		
	*new {
		arg
			numPlayers,
			projection = false;
	
		^super.new.init(numPlayers, projection);
	}
	
	init {
		arg
			numPlayers,
			projection;
		var
			bounds,
			w = 320, // width of a single KeyboardView
			h = 320 / 13.5 * 5; // height of a single KeyboardView		
		/*
		 *   The keyboards are arranged in the window like so:
		 *
		 *   +----------+----------+
		 *   | Player 0 | Player 1 |
		 *   +----------+----------+
		 *   | Player 2 | Player 3 |
		 *   +----------+----------+
		 *   | Player 4 | Player 5 |
		 *   +----------+----------+
		 *   | Player 6 | Player 7 |
		 *   +----------+----------+
		 *
		 *   N.B. This arrangement is the same regardless of the number of players.
		 */
		
		
		// Make window.
		if(
			projection,
			{
				bounds = Rect(1280 + 640, 0, 2 * w, 4 * h)
			},
			{
				bounds = Rect(640, 0, 2 * w, 4 * h)
			}
		);
		
		window = Window(
			"KeyboardWindow",
			bounds,
			resizable: false,
			border: false;
			
		);
		
		window.view.background = Color.black;
		window.front;
		CmdPeriod	.add({window.close});


		// Make keyboardViews.
		keyboardViews = Array.newClear(8);
		
		4.do{
			arg i;
			2.do{
				arg j;
				keyboardViews[2 * i + j] = KeyboardView(
					window,
					Rect(j * w, i * h, w, h)
				);
			}
		};
		

		try {
			this.init_osc_responders(numPlayers);
		}
		{
			"WARNING: KeyboardWindow couldn't find all the players in the network.".postln;
		};
		
	}
	
	init_osc_responders {
		arg numPlayers;

		// an OSCresponderNode for each player.
		// Players are "0.local", "1.local", "2.local", etc.
		oscResps = Array.newClear(numPlayers);

		numPlayers.do {
			arg i;
			oscResps[i] = OSCresponderNode(
				NetAddr(
					("%.local").format(i.asString),
					57120
				),
				'/keyboardWindow',
				{
					arg
						time,
						resp,
						msg;
					var
						playerNum,
						cmd,
						char;
					
					cmd = msg[1];
					char = msg[2].asString[0]; // expect a 1-character Symbol,
					                           // convert to a String (Array of Chars),
					                           // read the first Char					
					cmd.switch(
						\keyDown, {this.key_down(i, char)},
						\keyUp, {this.key_up(i, char)}
					);	
				}
			).add;
			
		// Remove the OSC responder when done.
		CmdPeriod	.add({oscResps[i].remove});
			
		}
	}
	
	key_down {
		arg playerNum, char;
		keyboardViews[playerNum].key_down(char);
	}
	
	key_up {
		arg playerNum, char;
		keyboardViews[playerNum].key_up(char);		
		
	}
}