MsgWindow {
	var
		<>public,
		<>appearanceDict,
		<>oscResp,
		<>clockText,
		<>msgText,
		<>timekeeper,
		<>counter,
		<>window;

	*new {
		arg
			playerNum = 0,
			public = false,
			projection = false,
			thisScreenWidth = 1280,
			projectionScreenWidth = 1280,
			displayHeight = 720;
			

		^super.newCopyArgs(public).init(
			playerNum,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight
		);
	}


	init {
		arg
			playerNum,
			projection,			
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight;
			
		var
			border;
							
		clockText = "--:--";
		msgText = "";

		/* Init OSC. */
		this.init_osc_responder(playerNum);

		/* Load appearance options. */
		if(
			playerNum == 0 && public,
			{
				if(
					projection,
					{
						appearanceDict = Dictionary[
							\bounds -> Rect(
								thisScreenWidth,
								0,
								projectionScreenWidth,
								displayHeight / 3
							),
							\font -> Font("Monaco", 12),
							\lineLength -> floor(projectionScreenWidth / 16),
							\numLines -> floor(displayHeight / 60)
		
						]
					},
					// Single-monitor display
					{
						appearanceDict = Dictionary[
							\bounds -> Rect(
								0,
								0,
								thisScreenWidth,
								displayHeight / 3
							),
							\font -> Font("Monaco", 12),
							\lineLength -> floor(thisScreenWidth / 16),
							\numLines -> floor(displayHeight / 60)
		
						]
					}
				);

			},
			{
				// Private window
				appearanceDict = Dictionary[
					\bounds -> Rect(0, 0, 320, 360),
					\font -> Font("Monaco", 10),
					\lineLength -> 40,
					\numLines -> 20
				]
			}
		);

		/* Set up window. */
		if(
			public,
			{
				border = false;
			},
			{
				border = true;
			}
		);
		
		window = Window.new(
			"MsgWindow",
			appearanceDict.at(\bounds),
			resizable: false,
			border: border
		);
		
		window.view.background = Color.black;
		window.front;
		CmdPeriod	.add({window.close});		

		window.drawHook = {
		
			Pen.font = appearanceDict.at(\font);
			Pen.fillColor = Color.green;

			// Draw clock.			
			if(
				not(public),
				{
					Pen.stringInRect(
						clockText,
						Rect(10, 10, window.view.bounds.width - 20, 20)
					)
				}
			);
			
			// Draw messages.
			Pen.stringInRect(
				msgText,
				Rect(10, 40, window.view.bounds.width - 20, window.view.bounds.height - 50)
			);
			
			Pen.fill;

		};
		
		// `timekeeper' is a rectangle that shifts position periodically
		// (see `advance_timekeeper' below).

		if(
			not(public),
			{
				counter = 4;
				timekeeper = UserView(window, this.set_timekeeper_bounds);
				timekeeper.background = Color.green;
			}
		);

	}

	set_timekeeper_bounds {
		if(
			public,
			{
				^Rect(7 * counter + 10, 27, 6, 12)
			},
			{
				^Rect(6 * counter + 10, 24, 5, 10)
			}
		);
		
	}

	init_osc_responder {
		arg
			playerNum;
		var
			conductorAddr,
			oscCmd,
			playerStr;

		conductorAddr = NetAddr("0.local", 57120);

		if(
			playerNum == 0 && public,
			{
				oscCmd = '/publicMsgWindow'
			},
			{
				oscCmd = '/msgWindow'
			}
		);

		oscResp = OSCresponderNode(
			conductorAddr,
			oscCmd,
			{
				arg
					time,
					resp,
					msg;

				msg[1].switch(
					\add_msg, {this.add_msg(msg[2].asString, msg[3].asString)},
					\set_clock, {this.set_clock(msg[2].asString)},
					\clear, {this.clear},
					\advance_timekeeper, {this.advance_timekeeper},
					\reset_timekeeper, {this.reset_timekeeper}
				);
			}
		).add;
		
		// Remove the OSC responder when done.
		CmdPeriod	.add({oscResp.remove});
	
	}

	add_msg {
		arg player, msg;
				
		msgText = msg.block_format(
			msgText,
			appearanceDict.at(\lineLength),
			appearanceDict.at(\numLines),
			"% > ".format(player.asString);
		);

		{window.refresh}.defer;
		
	}

	set_clock {
		arg time;

		// Assume it's already formatted
		clockText = time;
		{window.refresh}.defer;
		
	}

	clear {
		msgText = "";
		{window.refresh}.defer;
		
	}
	
	reset_timekeeper {
		counter = 4;
	}
	
	advance_timekeeper {
		counter = counter + 1 % 5;
		{timekeeper.bounds = this.set_timekeeper_bounds}.defer;

	}	
}
