PlorkChar {
	var
		<>voice,
		<>char,
		<>start,
		<>stop;

	*new {
		arg voice, char, start, stop;
		
		^super.newCopyArgs(voice, char, start, stop);
	}
	
	as_str {
		^voice.asString ++ "\t" ++ char.asString ++ "\t" ++ start.asString ++ "\t" ++ stop.asString ++ "\n"	}
	
}

PlorkCmd {
	var
		<>voice,
		<>time,
		<>freq,
		<>is_on;
		
	*new {
		arg
			voice,
			time,
			freq,
			is_on; // Python-style Boolean
			
		^super.newCopyArgs(voice, time, freq, is_on)
	}
	
	as_str {
		^voice.asString ++ "\t" ++ time.asString ++ "\t" ++ freq.asString ++ "\t" ++ is_on.asString ++ "\n"
	}
}

PlorkCmdList {
	var
		<>cmds;

	*new {
		^super.new.init;
	}

	init {
		cmds = List.new;
	}

	add {
		arg voice, time, freq, is_on;
		var lastIndex;
		lastIndex = cmds.size - 1;

		is_on.switch(
			false, {is_on = "0"},
			0,     {is_on = "0"},
			true,  {is_on = "1"},
			1,     {is_on = "1"}
		);
		
		if(
			freq == nil,
			{
				freq = cmds[lastIndex].freq
			}
		);
		if(
			is_on == nil,
			{
				is_on = cmds[lastIndex].is_on
			}
		);
		
		cmds.add(PlorkCmd(voice, time, freq, is_on))
	}
}
		
	



PlorkRecorder {
	var
		<>plorkBundle,
		<>voice,
		<>charFileName,
		<>cmdFileName,
		<>recordingName,
		
		
		<>conductorWindowResp,
		<>charWindowResp,
		<>msgWindowResp,
		<>allChars,       // A List of all chars so far
		                  // (updated whenever a command is *evaluated*).
		<>cmdChars,       // A List of chars for the current command.
		                  // These will disappear when the command is *executed*.
		<>stuckChars,     // A List of chars that will never disappear.
		<>plorkCmdList,
		<>recorder;

	*new {
		arg
			plorkBundle,
			voice,
			charFileName,
			cmdFileName,
			recordingName;
			
		^super.newCopyArgs(plorkBundle, voice, charFileName, cmdFileName, recordingName).init;
	}

	init {
		plorkCmdList = PlorkCmdList.new;
		plorkCmdList.add(voice, 0, 4410, 0);
		
		this.init_recorder;
		this.init_osc_responders;
		this.reset_charLists;		
	}


	init_recorder {
		Server.local.waitForBoot(
			{
				recorder = OutputRecorder(stereo: true, sixChannel: false);
			}
		)
	}

	reset_charLists {
		cmdChars = List.new;
		stuckChars = List.new;
		
	}

	update_chars {
		var newChars;
		
		newChars = cmdChars ++ stuckChars;
		
		newChars.do{
			arg i;
			i.as_str.post;
		};
		
		allChars = allChars ++ newChars;
		this.reset_charLists;
	
	}

	init_osc_responders {
		arg voice = 0;
		

		// Parse messages from the CodingWindow
		conductorWindowResp = OSCresponderNode(
			NetAddr(
				"0.local",
				57120
			),
			'/conductor',
			{
				arg
					time,
					resp,
					msg;
				var
					dict,
					eventTime,
					tmpCharList = List.new;


				dict = Parser(msg[1].asString).dict;
				if(
					dict.at("t") != nil, {
						eventTime = dict.at("t");
						if(
							plorkBundle.conductorBundle.conductorWindow.timing.valid_time(eventTime),
							{
							
								// Set the characters' stop time so that they
								// disappear when the command is executed.
								cmdChars.do{
									arg i;
									i.stop = eventTime;
								};
								
								// Add a command.
								plorkCmdList.add(voice, eventTime, dict.at(\impulseFreq), dict.at(\amp));
								
								
								this.update_chars;
							},
							{
								// all chars get stuck
								this.update_chars;
							}
						)
					},
					{
						// all chars get stuck
						this.update_chars;
					}
						
				)
			}
		).add;

		charWindowResp = OSCresponderNode(
			NetAddr(
				"0.local",
				57120
			),
			'/charWindow',
			{
				arg
					time,
					resp,
					msg;
				var
					char,
					clockTime,
					parser;
									
				char = msg[1].asString[0]; // expect a 1-character Symbol,
				                           // convert to a String (Array of Chars),
				                           // read the first Char

				clockTime = plorkBundle.conductorBundle.conductorWindow.timing.now;

				char.ascii.switch(
					// delete
					127, {
						stuckChars.add(cmdChars.pop);
					},
					{
						cmdChars.add(PlorkChar(voice, char, start: clockTime, stop: "inf"));
					}
					
					
				);
			}
		).add;
		
		msgWindowResp = OSCresponderNode(
			NetAddr(
				"0.local",
				57120
			),
			'/msgWindow',
			{
				arg
					time,
					resp,
					msg;
				var
					clockTime;

				msg[1].switch(
					\set_clock, {
						msg[2].asString.switch(
							"00:00", {
								this.pan_synth;
								{this.start_recording}.defer;
							},
							"--:--", {
								this.stop_recording;
								this.write_cmdFile;
								this.write_charFile;
							}
						)
					}
				);
			}
		).add;

		
		CmdPeriod	.add(
			{
				conductorWindowResp.remove;
				charWindowResp.remove;
				msgWindowResp.remove;
			}
		);
	}

	pan_synth {
		var
			panRange = ControlSpec(-1, 1),
			voiceRange = ControlSpec(0, 7);
			
		plorkBundle.playerBundle.synthWindow.synthDisplay.synth.synth.set(
			\pan,
			panRange.map(voiceRange.unmap(voice))
		)
	}
	
	start_recording {
		recorder.start_recording(recordingName);
	}
	
	stop_recording {
		recorder.stop_recording;
	}
	
	
	write_cmdFile {
		var file = File(cmdFileName, "w");
		plorkCmdList.cmds.do{
			arg i;
			file.write(i.as_str);
		};
		file.close;
	}
	
	
	write_charFile {
		var file = File(charFileName, "w");
		allChars.do{
			arg i;
			file.write(i.as_str);
		};
		file.close;
	}
	
}