Typers {
	var
		<>charWindow,
		<>conductorWindow,
		<>interleavedChars,
		<>timing,
		<>netAddr;

		
	*new {
		arg
			offset,
			cmdLists,
			charWindow,
			conductorWindow,
			duration;
			
		^super.newCopyArgs(
			charWindow,
			conductorWindow
		).init(offset, cmdLists, duration);
		
	}

	init {
		arg offset, cmdLists, duration;
		
		timing = Timing(duration).start(offset);

		// Rewrite the command lists as a single, interleaved string of characters.
		this.stringify_cmdLists(cmdLists);
		
		this.type_chars(offset);			
	}
	
	stringify_cmdLists {
		arg
			cmdLists;
		var
			stringLists,
			maxLength;
			
		stringLists = all {: "", i <- (1..cmdLists.size) };

		cmdLists.do {
			arg msgList, i;
			msgList.do {
				arg msg;
				stringLists[i] = stringLists[i] ++ msg ++ "_____";
			}
		};
		
		// Make all lines the same length.  Pad short lines with newline characters.
		maxLength = 0;
		stringLists.do {
			arg i;
			maxLength = max(maxLength, i.size);
		};
		
		stringLists.size.do {
			arg i;
			while(
				{ stringLists[i].size < maxLength },
				{ stringLists[i] = stringLists[i] ++ "_" }
			)
		};
		
		
		// Produce a list of interleaved characters.
		interleavedChars = List.new;
		maxLength.do {
			arg j;
			var theseChars = ""; // one char at a time from each string in stringLists
			
			stringLists.size.do {
				arg i;
				theseChars = theseChars ++ stringLists[i][j];
			};
			
			interleavedChars.add(theseChars);
		};
		
	
	}
	
	type_chars {
		arg
			offset;
		var
			noUnderlines,
			tempo = 5, // list items per second
			i;
			

		// Skip ahead to the appropriate point in the interleavedChars list.
		i = offset * tempo;
		
		Routine(
			{
				while(
					{i < interleavedChars.size},
					{
						noUnderlines = interleavedChars[i].replace("_", "");
						charWindow.add_str(noUnderlines);
						(1 / tempo).wait;
						i = i + 1;
					}
				)
			}
		).play;
	}
}