FreqTables {

	var
		<>freqs,
		<>amps,
		<>glitches,
		<>freqTable,
		<>glitchTable,
		<>ampTable;
		
	*new {
		arg freqs, amps;
		
		// Each amp corresponds to a freq;
		// freqs.size should equal amps.size.
		
		^super.newCopyArgs(freqs, amps).init;
	}

	init {
		var n = freqs.size + 1;
		
		glitches = all {: 10 * abs(4410 - i), i <- freqs};
		
		freqTable = Array2D.new(n, n);
		glitchTable = Array2D.new(n, n);
		ampTable = Array2D.new(n, n);
		
		this.reset_freqTables;
		this.reset_ampTable;
		
	}

	reset_freqTables {
		(0..freqs.size - 1).do{
			arg i;
			// upper row of all values
			freqTable[0, i] = freqs[i];
			glitchTable[0, i] = glitches[i];

			// right-hand column of all values
			freqTable[i+1, freqs.size] = freqs[i];
			glitchTable[i+1, freqs.size] = glitches[i];
		};
		
		
		// 110525: the glitch table now displays ratios
		(0..freqs.size - 2).do{
			arg i;
			((i + 1)..freqs.size - 1).do{
				arg j;
				var max_glitch, min_glitch;
				
				freqTable[i + 1, j] = abs(freqs[j] - freqs[i]); // difference of the two freqs
				glitchTable[i + 1, j] = Rational(glitches[j] / glitches[i]); // ratio of the two glitches
			}
		}	
	}
	
	reset_ampTable {
		(0..amps.size - 1).do{
			arg i;
			// upper row of all values
			ampTable[0, i] = amps[i];			

			// right-hand column of all values
			ampTable[i+1, freqs.size] = amps[i];
		};
		
		(0..amps.size - 2).do{
			arg i;
			((i + 1)..amps.size - 1).do{
				arg j;
				ampTable[i + 1, j] = abs(amps[j] * amps[i]);  // product of the two amps
			}
		}
	
	}

	set_freq {
		arg i, val;
		freqs[i] = val;
		glitches[i] = 10 * abs(4410 - val);
		this.reset_freqTables;
	}

	set_amp {
		arg i, val;
		amps[i] = val;
		this.reset_ampTable;
	}

}



