Key {
	var
		<>parent, // a Window
		<>bounds,
		<>downColor,
		<>upColor,
		<>view;
		
	*new {
		arg
			parent,
			bounds, // a Rect
			downColor = Color.green,
			upColor = Color.black;
			
		^super.newCopyArgs(
			parent,
			bounds,
			downColor,
			upColor
		).init;
	}

	init {

		view = SCUserView(
			parent,
			bounds
		);
	
		this.up;
	}
	
	up {
		{view.background = upColor}.defer;
	}
	
	down {
		{view.background = downColor}.defer;
	}	
}

KeyboardView {

	var
		<>parent,
		<>bounds,
		<>keyDict;
		
	*new {
		arg
			parent,
			bounds;
		^super.newCopyArgs(parent, bounds).init;
	}
	
	init {
		var
			x = bounds.width / 14.5,
			y = bounds.height / 5;		

		keyDict = Dictionary[
		
			[$`, $~] -> Rect(0, 0, x, y),
			[$1, $!] -> Rect(x, 0, x, y),
			[$2, $@] -> Rect(2 * x, 0, x, y),
			[$3, $#] -> Rect(3 * x, 0, x, y),
			[$4, $$] -> Rect(4 * x, 0, x, y),
			[$5, $%] -> Rect(5 * x, 0, x, y),
			[$6, $^] -> Rect(6 * x, 0, x, y),
			[$7, $&] -> Rect(7 * x, 0, x, y),
			[$8, $*] -> Rect(8 * x, 0, x, y),
			[$9, $(] -> Rect(9 * x, 0, x, y),
			[$0, $)] -> Rect(10 * x, 0, x, y),
			[$-, $_] -> Rect(11 * x, 0, x, y),
			[$=, $+] -> Rect(12 * x, 0, x, y),

			[$q, $Q] -> Rect(1.5 * x, y, x, y),
			[$w, $W] -> Rect(2.5 * x, y, x, y),
			[$e, $E] -> Rect(3.5 * x, y, x, y),
			[$r, $R] -> Rect(4.5 * x, y, x, y),
			[$t, $T] -> Rect(5.5 * x, y, x, y),
			[$y, $Y] -> Rect(6.5 * x, y, x, y),
			[$u, $U] -> Rect(7.5 * x, y, x, y),
			[$i, $I] -> Rect(8.5 * x, y, x, y),
			[$o, $O] -> Rect(9.5 * x, y, x, y),
			[$p, $P] -> Rect(10.5 * x, y, x, y),
			[$[, ${] -> Rect(11.5 * x, y, x, y),
			[$], $}] -> Rect(12.5 * x, y, x, y),
			[$\\, $|] -> Rect(13.5 * x, y, x, y), // N.B. double slash

			[$a, $A] -> Rect(1.75 * x, 2 * y, x, y),
			[$s, $S] -> Rect(2.75 * x, 2 * y, x, y),
			[$d, $D] -> Rect(3.75 * x, 2 * y, x, y),
			[$f, $F] -> Rect(4.75 * x, 2 * y, x, y),
			[$g, $G] -> Rect(5.75 * x, 2 * y, x, y),
			[$h, $H] -> Rect(6.75 * x, 2 * y, x, y),
			[$j, $J] -> Rect(7.75 * x, 2 * y, x, y),
			[$k, $K] -> Rect(8.75 * x, 2 * y, x, y),
			[$l, $L] -> Rect(9.75 * x, 2 * y, x, y),
			[$;, $:] -> Rect(10.75 * x, 2 * y, x, y),
			[$', $"] -> Rect(11.75 * x, 2 * y, x, y),

			[$z, $Z] -> Rect(2.25 * x, 3 * y, x, y),
			[$x, $X] -> Rect(3.25 * x, 3 * y, x, y),
			[$c, $C] -> Rect(4.25 * x, 3 * y, x, y),
			[$v, $V] -> Rect(5.25 * x, 3 * y, x, y),
			[$b, $B] -> Rect(6.25 * x, 3 * y, x, y),
			[$n, $N] -> Rect(7.25 * x, 3 * y, x, y),
			[$m, $M] -> Rect(8.25 * x, 3 * y, x, y),
			[$,, $<] -> Rect(9.25 * x, 3 * y, x, y),
			[$., $>] -> Rect(10.25 * x, 3 * y, x, y),
			[$/, $?] -> Rect(11.25 * x, 3 * y, x, y),

			[$ ] -> Rect(4.25 * x, 4 * y, 5 * x, y)
		];
		
		// Swap out Rect values of keyDict for Key objects.
		keyDict.keys.do{
			arg char;
			keyDict.put(
				char,
				Key(
					parent,
					keyDict.at(char).moveBy(bounds.left, bounds.top)
				)
			) 
		};
		
	}
	
	key_down {
		arg char;
		keyDict.keys.do{
			arg i;
			if(
				i.indexOf(char) != nil,
				{
					keyDict.at(i).down;
				}
			)
		};	
	}

	key_up {
		arg char;
		keyDict.keys.do{
			arg i;
			if(
				i.indexOf(char) != nil,
				{
					keyDict.at(i).up;
				}
			)
		};
	}

}