Parser {
	var
		<>str,
		<>dict; // Dictionary containing parsed values
		
	*new {
		arg str;
		^super.newCopyArgs(str).init;
	}
	
	init {
		var tokens, word, msg;

		dict = Dictionary[
			"t" -> nil,
			\amp -> nil,
			\impulseFreq -> nil,
			"m" -> nil,
			"M" -> nil
		];
		
		// Tokenize the string.
		tokens = this.tokenize;
				
		// Extract messages.
		while(
			{tokens.size > 0},
			{
				word = tokens.removeAt(0);

				// timing?
				if(
					word.is_valid_time_format,
					{
						dict.put("t", word.to_time_float);
					}
				);
				
				// amp?
				if(
					word == "+",
					{
						dict.put(\amp, 1);
					}
				);
				
				if(
					word == "-",
					{
						dict.put(\amp, 0);
					}
				);
				
				// frequency?
				if(
					word.is_valid_number,
					{
						dict.put(\impulseFreq, word.asFloat);
					}
				);
				
				// message?
				if(
					word.asSymbol == \m,
					{
						msg = tokens.removeAt(0);
						dict.put("m", msg);
					}
				);

				if(
					word.asSymbol == \M,
					{
						
						msg = tokens.removeAt(0);
						dict.put("M", msg);
					}
				)
			}
		)
	}				
				

	tokenize {
		var quoteSplit, tokens;

		/*   Split string by quotes.
		 *   The list that's returned should always have an odd number of entries
		 *   (including empty strings),
		 *   because there should always be an even number of quotation marks.
		 */
		
		quoteSplit = str.split($");
		
		if(
			quoteSplit.size.even,
			{
				Error("Bad string formatting: I count an odd number of quotation marks.").throw;
			}
		);
		

		/*   The quoted strings have odd indicies.
		 *   Split all the entries with even indicies by spaces.
		 */
		
		tokens = List.new;
		
		quoteSplit.size.do{
			arg i;
			var spaceSplit;
			if(
				i.even,
				{
					spaceSplit = quoteSplit[i].split($ );
					spaceSplit.do{
						arg item;
						if(
							item != "",
							{
								tokens.add(item);
							}
						)
					}
				},
				{
					tokens.add(quoteSplit[i])
				}
			)
		};
		
		^tokens
	}	
}