SimulationBundle {
	var
/*		<>keyboardWindow,
*/
		<>charWindow,
		<>freqTablesWindow,
		<>msgWindow,
		<>conductorWindow,
		<>typers;
	
	*new {
		arg
			offset = 0,
			projection = false,
			thisScreenWidth = 1280,
			projectionScreenWidth = 1280,
			displayHeight = 720,			
			msgLists = nil;

		^super.new.init(
			offset,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight,
			msgLists
		);
	}
	
	init {
		arg
			offset,
			projection,
			thisScreenWidth,
			projectionScreenWidth,
			displayHeight,
			msgLists;
		var
			soundEvents,
			publicMsgs,
			duration = 410.2;

		// Hide the ConductorWindow behind the other windows.
		conductorWindow = ConductorWindow(
			numPlayers: 1,
			simulation: true,
			duration: duration
		);

/*
		keyboardWindow = KeyboardWindow(
			numPlayers: 1,
			projection: projection
		);
*/
		charWindow = CharWindow(
			numPlayers: 1,
			projection: projection,
			thisScreenWidth: thisScreenWidth,
			projectionScreenWidth: projectionScreenWidth,
			displayHeight: displayHeight			
		);

		freqTablesWindow = FreqTablesWindow(
			numPlayers: 8,
			projection: projection,
			thisScreenWidth: thisScreenWidth,
			projectionScreenWidth: projectionScreenWidth,
			displayHeight: displayHeight			
		);
		msgWindow = MsgWindow(
			playerNum: 0,
			public: true,
			projection: projection,
			thisScreenWidth: thisScreenWidth,
			projectionScreenWidth: projectionScreenWidth,
			displayHeight: displayHeight	
		);
		

		if(
			msgLists == nil,
			{
				soundEvents = this.load_sound_events;
				msgLists = all {: [], i <- (0..7) };
				soundEvents.do {
					arg i;
					msgLists = msgLists.cat_entries(i.as_msg_lists);
				};
				
/*				// prepend public messages
				msgLists = this.load_public_msgs.cat_entries(msgLists);

				// 110208: no more time-perception-warping public messages
*/
			}
		);

		Server.local.waitForBoot(
			{
			
				conductorWindow.start_clock(offset);
				
				// Start typing.
				typers = Typers(
					offset,
					msgLists,
					charWindow,
					conductorWindow,
					duration
				);

				// Pre-load all the messages.
				conductorWindow.parse_msg_lists(msgLists);				
			};
		)
	}
	
	load_sound_events {
		^[
			Chord(
				startTimes: 200 ! 8,
				endTimes: nil ! 8,
				playerNums: (0..7),
				freqs: [4410, 4411, 4411.1, 4412, 4412.01, 4413.1, 4414.01, 4414.1]
			),
			Hocket(
				startTime: 230,
				endTime: 231,
				freqRange: [4410, 4420],
				playerOrder: (0..7).scramble,
				evenness: 0.8,
				ampString: ["-", "+"],
				changeFreq: [false, true]
			),
			Hocket(
				startTime: 245,
				endTime: 247,
				freqRange: [4410, 4420],
				playerOrder: (0..7).scramble,
				evenness: 0.1,
				ampString: ["", "-"],
				changeFreq: [true, false]
			),
			Hocket(
				startTime: 260,
				endTime: 260.5,
				freqRange: [4410, 4410.1],
				playerOrder: (0..7).scramble[0..5],
				evenness: 1,
				ampString: ["", ""],
				changeFreq: [true, false]
			),
			Hocket(
				startTime: 265,
				endTime: 270,
				freqRange: [4410, 4410.1],
				playerOrder: (0..7).scramble[0..4],
				evenness: 0,
				ampString: ["+", ""],
				changeFreq: [false, false]
			),
			Polyrhythms(
				startTimes: 248 ! 5,
				endTimes: 250 ! 5,
				playerNums: (0..7).scramble[0..4],
				numPulses: [2, 7, 11, 13, 17]
			),
			ChordProgression(
				startTime: 272,
				endTime: 310,
				playerNums: [0, 2, 3, 7],
				numerals: [1, 4, 5, 6, 4, 5]
			),
			ChordProgression(
				startTime: 300,
				endTime: 301,
				playerNums: [1, 4, 5, 6],
				numerals: [4, 5, 4, 5, 6, 5, 6, 4, 5, 6]
			),
			Polyrhythms(
				startTimes: 309.7 ! 6,
				endTimes: 310.2 ! 6,
				playerNums: (0..7).scramble[0..5],
				numPulses: [2, 3, 4, 5, 6, 8]
			)
		];	
	}
	
/*	load_public_msgs {

		^[
			[
				"5:00 M \"The piece ends with a 2:3:4:5:6:8 polyrhythm " ++
				"between 6:49.7 and 6:50.2.\""
			],
			[
				"5:17 M \"A low-frequency chord progression IV V IV V vi V " ++
				"vi IV V vi lasts from 6:40 to 6:41.\""
			],
			[
				"5:43 M \"The next message about chords appears at a time that, " ++
				"measured in seconds, is a prime number.\""
			],
			[
				"6:13 M \"A low-frequency chord progression I IV V vi IV V " ++
				"lasts from 6:12 to 6:50.\""
			],
			[
				"6:17 M \"At 6:17, a message about itself appears.\""
			],
			[
				"6:22.1 M \"A 2:7:11:13:17 polyrhythm lasts from 5:48 to 5:50.\""
			],
			[
				"6:47 M \"Between 5:30 and 5:31, all players switch off, then " ++
				"switch back on to a random frequency between 4410 and 4420.\""
			],
			[
				"6:48 M \"The piece begins at 5:00 with a chord made up of frequencies that are " ++
				"integers between 4410 and 4414 plus a tenth, hundredth, or nothing.\""
			]
		]
	}
*/
}